package net.cattaka.rdbassistant.core;

import java.math.BigDecimal;
import java.sql.Date;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Time;
import java.sql.Timestamp;

import net.cattaka.rdbassistant.sql.ColumnConverter;
import net.cattaka.rdbassistant.sql.DateColumnConverter;
import net.cattaka.rdbassistant.sql.DateTimeColumnConverter;
import net.cattaka.rdbassistant.sql.BigDecimalColumnConverter;
import net.cattaka.rdbassistant.sql.OtherColumnConverter;
import net.cattaka.rdbassistant.sql.StringColumnConverter;
import net.cattaka.rdbassistant.sql.TimeColumnConverter;
import net.cattaka.rdbassistant.sql.TimestampColumnConverter;

public class RdbaConnectionUtil {
	/**
	 * 
	 * @param rdbaConnection
	 * @param rsmd
	 * @return
	 * @throws SQLException
	 */
	public static ColumnConverter[] createColumnConverterList(RdbaConnection rdbaConnection, ResultSetMetaData rsmd) throws SQLException {
		ColumnConverter[] results = new ColumnConverter[rsmd.getColumnCount()];
		for (int i=0;i<results.length;i++) {
			Class<?> columnClass = rdbaConnection.getClassFromSqlType(rsmd.getColumnType(i+1));
			if (columnClass == String.class) {
				results[i] = new StringColumnConverter(rsmd.getPrecision(i+1));
			} else if (columnClass == BigDecimal.class) {
				results[i] = new BigDecimalColumnConverter(rsmd.getPrecision(i+1), rsmd.getScale(i+1));
			} else if (columnClass == Timestamp.class) {
				results[i] = new TimestampColumnConverter();
			} else if (columnClass == Time.class) {
				results[i] = new TimeColumnConverter();
			} else if (columnClass == Date.class) {
				results[i] = new DateColumnConverter();
			} else if (columnClass == java.util.Date.class) {
				results[i] = new DateTimeColumnConverter();
			} else {
				results[i] = new OtherColumnConverter();
			}
		}
		
		return results;
	}
}
